@@ -12,7 +12,7 @@ from . import tyming
 from ..help import timing, helping
 
 
-Deed = namedtuple("Deed", "dog retyme index")
+Deed = namedtuple("Deed", "dog retyme doer")
 
 class Doist(tyming.Tymist):
     """
@@ -22,11 +22,11 @@ class Doist(tyming.Tymist):
     .tick method.
     The doist may treat .tyme as artificial time or synchonize it to real time.
 
-    .enter method prepares deeds deque of triples (dog, retyme, index) where
+    .enter method prepares deeds deque of triples (dog, retyme, doer) where
         dog is a doer generator returned by calling doer generator instances,
         functions, or methods.
 
-    .recur method runs its deeds deque of triples (dog, retyme, index) once per
+    .recur method runs its deeds deque of triples (dog, retyme, doer) once per
         invocation.
         This synchronizes their cycle time .tyme to the Doist's tyme.
 
@@ -44,11 +44,11 @@ class Doist(tyming.Tymist):
         doers (list): Doer class instances, generator methods or
                 function callables with attributes tock, done, and opts dict().
                 Used throughout the execution lifecycle.
-        deeds (deque): Tuples of form (dog, retyme, index). Where:
-            dog is generator
+        deeds (deque): Tuples of form (dog, retyme, doer). Where:
+            dog is generator created by doer
             retyme is tyme (real or simulated) in seconds when dog should run next
-            index is position of associated doer in .doers list used to assign
-                .done state to associated doer for dog
+            doer is associated doer in .doers list used to assign its .done state
+                given completion state of its dog
             Used throughout the execution lifecycle. The normal
             case is use the default empty initialization performed here and
             update in .enter().
@@ -64,7 +64,7 @@ class Doist(tyming.Tymist):
         .tick increments .tyme by one .tock or provided tock
 
     Methods:
-        .enter prepare deeds, deque of triples (dog, retyme, index)
+        .enter prepare deeds, deque of triples (dog, retyme, doer)
         .recur  run through all deeds once
         .do repeadedly call .recur until all dogs in deeds are complete or
             times out do to reaching time limit
@@ -102,11 +102,11 @@ class Doist(tyming.Tymist):
         """
         Readies deeds deque from .doers or doers if any and then iteratively
         runs .recur over deeds deque until completion of all deeds.
-        Each entry in deeds is a triple (dog, retyme, index)  where:
+        Each entry in deeds is a triple (dog, retyme, doer)  where:
             dog is generator
             retyme is tyme (real or simulated) in seconds when dog should run next
-            index is position of associated doer in .doers list used to assign
-                .done state to associated doer for dog
+            doer is from .doers list used to assign its .done state given
+            associated completion state of its dog
 
         If interrupted by exception call .close on each dog to force exit context.
 
@@ -175,17 +175,17 @@ class Doist(tyming.Tymist):
                     raise
 
         finally: # finally clause always runs regardless of exception or not.
-            self.close()  # force close remaining deeds throws GeneratorExit
+            self.exit()  # force close remaining deeds throws GeneratorExit
 
 
     def enter(self, doers=None):
         """
         Enter context
-        Returns (deque):  deeds deque of triples (dog, retyme, index)  where:
+        Returns (deque):  deeds deque of triples (dog, retyme, doer)  where:
             dog is generator
             retyme is tyme (real or simulated) in seconds when dog should run next
-            index is position of associated doer in .doers list used to assign
-                .done state to associated doer for dog
+            doer is from .doers list used to assign its .done state given
+                completion state of its dog
 
         Calls each generator callable (instance or function or method) in .doers
         to create each generator dog. Injects own tymth function closure, and
@@ -204,7 +204,7 @@ class Doist(tyming.Tymist):
 
         Returns:
             deeds deque():
-                A deed is tuple of form (dog, retyme, index). If not provided
+                A deed is tuple of form (dog, retyme, doer). If not provided
                 uses .deeds.
 
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
@@ -216,7 +216,7 @@ class Doist(tyming.Tymist):
         else:
             deeds = deque()  # when doers is provided then don't use .deeds
 
-        for index, doer in enumerate(doers):
+        for doer in doers:
             try:
                 doer.done = None  #  None before enter. enter may set to False
             except AttributeError:  # when using bound method for generator function
@@ -226,71 +226,69 @@ class Doist(tyming.Tymist):
             dog = doer(tymth=self.tymen(), tock=doer.tock, **opts)
             try:
                 next(dog)  # run enter by advancing to first yield
-            except StopIteration:
+            except StopIteration as ex:
+                try:
+                    doer.done = ex.value if ex.value else False  # assign done state
+                except AttributeError:
+                    doer.__func__.done = ex.value if ex.value else False  # assign done state
                 continue  # don't append
-            deeds.append((dog, self.tyme, index))
+            deeds.append((dog, self.tyme, doer))
         return deeds
 
 
-    def recur(self, doeds = None):
+    def recur(self, deeds = None):
         """
         Recur once through deeds deque of tuples (triples) of form
-        (dog, retyme, index) and update in place
+        (dog, retyme, doer) and update in place
 
-        Each deed is deque of tuples of  form (dog, retyme, index) where:
+        Each deed is deque of tuples of  form (dog, retyme, doer) where:
             dog is generator
             retyme is tyme (real or simulated) in seconds when dog should run next
-            index is position of associated doer in .doers list used to assign
-                .done state to associated doer for dog
+            doer is from .doers list used to assign its .done state given
+            associated completion state of its  dog
 
         Each cycle checks all generators in deeds deque and runs if retyme past.
         At end of cycle advances .tyme by one .tock by calling .tick()
 
         Parameters:
-            doeds (tuple): optional of form (doers list, deeds deque)
-                If not provided uses .doers and .deeds.
-                Where:
-                    doers (list):  of  generator method or function callables
-                    with attributes  tock, done, and opts dict().
-
-                    deeds (deque): tuples of form (dog, retyme, index).
+            deeds (deque):  tuples of form (dog, retyme, doer).
                     Parameterization here of deeds enables some special cases.
 
-        The Parameterization here of doers and deeds enables some special cases
+        The Parameterization here of deeds enables some special cases
         such as manual testing or iteraton.
         The normal case is to initialize .doers in .__init__. or .do() and to
         initialize .deeds in .__init__. and then update in .enter()
         """
-        if doeds is not None:
-            doers = doeds[0]
-            deeds = doeds[1]
-        else:
-            doers = self.doers
+        if deeds is None:
             deeds = self.deeds
 
-        for i in range(len(deeds)):  # iterate once over each deed
-            dog, retyme, index = deeds.popleft()  # pop it off
+        deeds.append((None, None, None))  # append run through once marker
+        while deeds: # do while uses explicit break to exit while
+            dog, retyme, doer = deeds.popleft()  # pop it off
+            if not dog:  # Marker detected so this run through once has completed
+                break  # break loop at marker signifies once through
 
             if retyme <= self.tyme:  # run it now
-                try:
-                    tock = dog.send(self.tyme)  # send tyme. yield tock
+                try:  # send tyme. yield tock, tock may change during sended run
+                    tock = dog.send(self.tyme)  # yielded tock == 0.0 means re-run asap
                 except StopIteration as ex:  # returned instead of yielded
-                    doer = doers[index]
                     try:
                         doer.done = ex.value if ex.value else False  # assign done state
                     except AttributeError:   # when using bound method for generator function
                         doer.__func__.done = ex.value if ex.value else False  # assign done state
                 else:  # reappend for next pass
-                    if tock is None:  # bare yield returns None
-                        tock = 0.0  # rerun asap when tock == 0.0
-                    deeds.append((dog, retyme + tock, index))  # tock may change during run
+                    if not tock:  # tock is None or tock == 0.0 with empty yield tock == None
+                        retyme = self.tyme + self.tock  # rerun at next recur
+                    else:
+                        retyme += tock  # cumulative retyme of doer tock
+                    deeds.append((dog, retyme, doer))  # reappend for next run through
             else:  # not retyme yet
-                deeds.append((dog, retyme, index))  # reappend for next pass
+                deeds.append((dog, retyme, doer))  # reappend for next pass
 
         self.tick()  # advance .tyme by one doist .tock
 
 
-    def close(self, doeds = None):
+    def exit(self, deeds = None):
         """
         Force exit each still opened deed calling .close on the dog generator
         which throws a GeneratorExit to the generator.
@@ -310,29 +308,22 @@ class Doist(tyming.Tymist):
             exit A
 
         Parameters:
-            doeds (tuple): optional of form (doers list, deeds deque)
-                If not provided uses .doers and .deeds.
-                Where:
-                    doers (list):  of  generator method or function callables
-                    with attributes  tock, done, and opts dict().
-                    deeds (deque): tuples of form (dog, retyme, index).
-                    Parameterization here of deeds enables some special cases.
+            deeds (deque): tuples of form (dog, retyme, doer).
+                If not provided uses .deeds.
+                Parameterization here of deeds enables some special cases.
         """
-        if doeds is not None:
-            doers = doeds[0]
-            deeds = doeds[1]
-        else:
-            doers = self.doers
+        if deeds is None:
             deeds = self.deeds
 
         while(deeds):  # .close each remaining dog in deeds in reverse order
-            dog, retime, index = deeds.pop() #pop it off
+            dog, retime, doer = deeds.pop()  # pop it off in reverse (right side)
+            if not dog:  # marker deed
+                continue  # skip marker
             try:
                 tock = dog.close()  # force GeneratorExit. Maybe log exit tock tyme
             except StopIteration:
                 pass  # Hmm? Not supposed to happen!
             else:  # set done state
-                doer = doers[index]
                 try:
                     doer.done = False  # forced close
                 except AttributeError:  # when using bound method for generator function
@@ -348,13 +339,8 @@ class Doist(tyming.Tymist):
             doers is list of doers to add as extension.
 
         """
-        deeds = self.enter(doers=doers)  # provide fresh deeds
-        offset = len(self.doers)  # get offset of index for extending dogs
-        for i in range(len(deeds)):  # iterate once over each deed
-            dog, retyme, index = deeds.popleft()
-            index += offset
-            deeds.append((dog, retyme, index))
-
+        doers = [doer for doer in doers if doer not in self.doers] # ensure unique
+        deeds = self.enter(doers=doers)  # provide fresh deeds for new doers
         self.doers.extend(doers)
         self.deeds.extend(deeds)
 
@@ -368,34 +354,22 @@ class Doist(tyming.Tymist):
             doers is list of doers to remove.
 
         """
-        rdoers = list(doers)  # doers to remove, make copy since destructive
-        indices = {}  # map indices for each doer as appears in .doers to rdoers
-        j = 0  # index in remove list
-        for doer in list(rdoers):  # make copy since inplace filter of rdoers
-            try:
-                i = self.doers.index(doer)  # make sure a member of .doers
-            except ValueError:
-                rdoers.remove(doer)  # not in .doers so ignore remove from rdoers
-            else:
-                indices[i] = j
-                j += 1
-
+        rdoers = [doer for doer in doers if doer in self.doers] # ensure in .doers
         rdeeds = deque()  # fresh deque for deeds to remove
         deeds = self.deeds  # edit update self.deeds in place
-        k = 0  # reappended index of remaining (not removed) deed
         for i in range(len(deeds)):  # iterate once over each deed
-            dog, retyme, index = deeds.popleft()
-            if index in indices:  # found deed to remove and close
-                rdeeds.append((dog, retyme, indices[index]))  # add to removal deque
+            dog, retyme, doer = deeds.popleft()
+            if not dog:  # reappend the run through once marker deed
+                deeds.append((dog, retyme, doer))
+            elif doer in rdoers:  # found deed to remove and close
+                rdeeds.append((dog, retyme, doer))  # add to removal deque
             else:  # keep deed do not remove and close
-                deeds.append((dog, retyme, k))  # reappend
-                k += 1
+                deeds.append((dog, retyme, doer))  # reappend
 
         for doer in rdoers:  # update .doers to remove rdoers
             self.doers.remove(doer)
 
-        self.close(doeds=(rdoers, rdeeds))  # only close the removed ones
-
+        self.exit(deeds=rdeeds)
 
 
 def doify(f, name=None, tock=0.0, **opts):
@@ -778,10 +752,10 @@ class DoDoer(Doer):
     Properties:
         doers (list): Doer or Doist compatible generator instances,
             functions, or methods.
-        deeds (deque):  tuples of form (dog, retyme, index)  where:
-            dog is generator
-            retyme is tyme in seconds when next should run may be real or simulated
-            index is position of associated doer in .doers list
+        deeds (deque):  tuples of form (dog, retyme, doer)  where:
+            dog is generator created by doer.
+            retyme is tyme in seconds when next should run may be real or simulated.
+            doer is associated doer in .doers list.
             Used throughout the execution lifecycle. The normal
             case is use the default empty initialization performed here and
             update in .enter().
@@ -838,8 +812,6 @@ class DoDoer(Doer):
                 are complete. Enables dynamically managing extending or removing
                 doers and associated deeds while running.
 
-        In each deed the index must match offset of the doer in the doers list.
-
         """
         super(DoDoer, self).__init__(**kwa)
         self.doers = list(doers) if doers is not None else []
@@ -871,7 +843,7 @@ class DoDoer(Doer):
     def deeds(self):
         """
         deeds property getter, get ._deeds
-        .deeds is deque of triples, each of form (dog, retyme, index)
+        .deeds is deque of triples, each of form (dog, retyme, doer)
         """
         return self._deeds
 
@@ -926,8 +898,6 @@ class DoDoer(Doer):
                 doers and associated deeds while running.
                 When not provided use .always.
             opts is dict of injected optional additional parameters
-
-        In each deed the index must match offset of the doer in the doers list.
         """
         always = always if always is not None else self.always
         if doers is not None:
@@ -968,10 +938,10 @@ class DoDoer(Doer):
         """
         Do 'enter' context actions. Equivalent of Doist.enter()
 
-        Returns deeds deque of triples (dog, retyme, index)  where:
-            dog is generator
+        Returns deeds deque of triples (dog, retyme, doer)  where:
+            dog is generator created by doer
             retyme is tyme in seconds when next should run may be real or simulated
-            index is position of associated doer in .doers list
+            doer is doer for dog from doers list
 
         Calls each generator callable (function or method) in .doers to create
         each generator dog.
@@ -987,11 +957,9 @@ class DoDoer(Doer):
 
         Returns:
             deeds deque():
-                A deed is tuple of form (dog, retyme, index). If not provided
+                A deed is tuple of form (dog, retyme, doer). If not provided
                 uses .deeds.
 
-        In each deed the index must match offset of the doer in the doers list.
-
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
         For setting attributes on bound methods.
         """
@@ -1001,7 +969,7 @@ class DoDoer(Doer):
         else:
             deeds = deque()
 
-        for index, doer in enumerate(doers):
+        for doer in doers:
             try:
                 doer.done = None  # None before enter. enter may set to False
             except AttributeError:   # when using bound method for generator function
@@ -1010,28 +978,28 @@ class DoDoer(Doer):
             dog = doer(tymth=self.tymth, tock=doer.tock, **opts)
             try:
                 next(dog)  # run enter by advancing to first yield
-            except StopIteration:
-                continue  # don't append
-            deeds.append((dog, self.tyme, index))
+            except StopIteration as ex:  # returned instead of yielded
+                try:
+                    doer.done = ex.value if ex.value else False  # assign done state
+                except AttributeError:
+                    doer.__func__.done = ex.value if ex.value else False  # assign done state
+                continue  # don't append already complete
+            deeds.append((dog, self.tyme, doer))
         return deeds
 
 
-    def recur(self, tyme, doeds = None):
+    def recur(self, tyme, deeds = None):
         """
         Do 'recur' context actions. Equivalent of Doist.recur
 
         Parameters:
-            tyme (float): is output of send fed to do yield, Doist feeds its .tyme
-                because tymist is injected by doist or dodoer doing this dodoer
-                self.tyme is same as tyme.
-             doeds (tuple): optional of form (doers (ist, deeds deque)
-                If not provided uses .doers and .deeds.
-                Where:
-                    doers (list):  of  generator method or function callables
-                    with attributes  tock, done, and opts dict().
-
-                    deeds (deque): tuples of form (dog, retyme, index).
-                    Parameterization here of deeds enables some special cases.
+            tyme (float): is output of send fed to do yield, The root scheduler
+                Doist feeds its .tyme which propagates down the chain of DoDoers
+                Because tymist is injected by doist or dodoer, self.tyme is same
+                as tyme. So may use either which is more convenient.
+             deeds (deque): tuples of form (dog, retyme, doer).
+                If not provided uses .deeds.
+                Parameterization here of deeds enables some special cases.
 
         Returns completion state of recurrence actions.
            True means done False means continue
@@ -1040,68 +1008,59 @@ class DoDoer(Doer):
 
         Each cycle checks all generators dogs in deeds deque and runs if retyme past.
         """
-        if doeds is not None:
-            doers = doeds[0]
-            deeds = doeds[1]
-        else:
-            doers = self.doers
+        if deeds is None:
             deeds = self.deeds
 
-        for i in range(len(deeds)):  # iterate once over each deed
-            dog, retyme, index = deeds.popleft()  # pop it off
+        deeds.append((None, None, None))  # append run through once marker
+        while deeds:  # do while uses explicit break to exit while
+            dog, retyme, doer = deeds.popleft()  # pop it off
+            if not dog:  # Marker detected so this run through once has completed
+                break  # break loop at marker signifies once through
 
             if retyme <= tyme:  # run it now
-                try:  # tock == 0.0 means re-run asap
-                    tock = dog.send(tyme)  # send tyme. yield tock
+                try:  # send tyme. yield tock, tock may change during sended run
+                    tock = dog.send(tyme)  # yielded tock == 0.0 means re-run asap
                 except StopIteration as ex:  # returned instead of yielded
-                    doer = doers[index]
                     try:
                         doer.done = ex.value if ex.value else False  # assign done state
                     except AttributeError:
                         doer.__func__.done = ex.value if ex.value else False  # assign done state
                 else:  # reappend for next pass
-                    if tock is None:  # bare yield results None
-                        tock = 0.0  # # rerun asap when tock == 0.0
-                    deeds.append((dog, retyme + tock, index))  # tock may change during run
+                    if not tock:  # tock is None or tock == 0.0 with empty yield tock == None
+                        retyme = tyme + self.tock  # rerun at next recur
+                    else:
+                        retyme += tock  # cumulative retyme of doer tock
+                    deeds.append((dog, retyme, doer))  # reappend for next run through
             else:  # not retyme yet
-                deeds.append((dog, retyme, index))  # reappend for next pass
+                deeds.append((dog, retyme, doer))  # reappend for next run through
 
         return (not deeds)  # True if deeds deque is empty
 
 
-    def exit(self, doeds = None):
+    def exit(self, deeds = None):
         """
         Do 'exit' context actions.
 
         Parameters:
-            doeds (tuple): optional of form (doers list, deeds deque)
-                If not provided uses .doers and .deeds.
-                Where:
-                    doers (list):  of  generator method or function callables
-                    with attributes  tock, done, and opts dict().
-                    deeds (deque): tuples of form (dog, retyme, index).
-                    Parameterization here of deeds enables some special cases.
-
-        In each deed the index must match offset of the doer in the doers list.
+            deeds (deque): of deed tuples of form (dog, retyme, doer)
+                If not provided uses .deeds.
+                Parameterization here of deeds enables some special cases.
 
         See: https://stackoverflow.com/questions/40528867/setting-attributes-on-func
         For setting attributes on bound methods.
         """
-        if doeds is not None:
-            doers = doeds[0]
-            deeds = doeds[1]
-        else:
-            doers = self.doers
+        if deeds is None:
             deeds = self.deeds
 
         while(deeds):  # .close each remaining dog in deeds in reverse order
-            dog, retime, index = deeds.pop()  # pop it off in reverse (right side)
+            dog, retime, doer = deeds.pop()  # pop it off in reverse (right side)
+            if not dog:  # marker deed
+                continue  # skip marker
             try:
                 tock = dog.close()  # force GeneratorExit
             except StopIteration:
                 pass  # Hmm? Not supposed to happen!
             else:  # set done state
-                doer = doers[index]
                 try:
                     doer.done = False  # forced close
                 except AttributeError:  # when using bound method for generator function
@@ -1117,13 +1076,8 @@ class DoDoer(Doer):
             doers is list of doers to add as extension.
 
         """
-        deeds = self.enter(doers=doers)  # provide fresh deeds
-        offset = len(self.doers)  # get offset of index for extending dogs
-        for i in range(len(deeds)):  # iterate once over each deed
-            dog, retyme, index = deeds.popleft()
-            index += offset
-            deeds.append((dog, retyme, index))
-
+        doers = [doer for doer in doers if doer not in self.doers] # ensure unique
+        deeds = self.enter(doers=doers)  # provide fresh deeds for new doers
         self.doers.extend(doers)
         self.deeds.extend(deeds)
 
@@ -1137,33 +1091,22 @@ class DoDoer(Doer):
             doers is list of doers to remove.
 
         """
-        rdoers = list(doers)  # doers to remove, make copy since destructive
-        indices = {}  # map indices for each doer as appears in .doers to rdoers
-        j = 0  # index in remove list
-        for doer in list(rdoers):  # make copy since inplace filter of rdoers
-            try:
-                i = self.doers.index(doer)  # make sure a member of .doers
-            except ValueError:
-                rdoers.remove(doer)  # not in .doers so ignore remove from rdoers
-            else:
-                indices[i] = j
-                j += 1
-
+        rdoers = [doer for doer in doers if doer in self.doers] # ensure in .doers
         rdeeds = deque()  # fresh deque for deeds to remove
         deeds = self.deeds  # edit update self.deeds in place
-        k = 0  # reappended index of remaining (not removed) deed
         for i in range(len(deeds)):  # iterate once over each deed
-            dog, retyme, index = deeds.popleft()
-            if index in indices:  # found deed to remove and close
-                rdeeds.append((dog, retyme, indices[index]))  # add to removal deque
+            dog, retyme, doer = deeds.popleft()
+            if not dog:  # reappend the run through once marker deed
+                deeds.append((dog, retyme, doer))
+            elif doer in rdoers:  # found deed to remove and close
+                rdeeds.append((dog, retyme, doer))  # add to removal deque
             else:  # keep deed do not remove and close
-                deeds.append((dog, retyme, k))  # reappend
-                k += 1
+                deeds.append((dog, retyme, doer))  # reappend
 
         for doer in rdoers:  # update .doers to remove rdoers
             self.doers.remove(doer)
 
-        self.exit(doeds=(rdoers, rdeeds))
+        self.exit(deeds=rdeeds)
 
 
 def bareDo(tymth=None, tock=0.0, **opts):
