@@ -20,19 +20,17 @@ import ssl
 from collections import deque
 from contextlib import contextmanager
 
-
 from ... import help
 from ...base import tyming, doing
 from .. import coring
 
-
 logger = help.ogler.getLogger()
 
 
 @contextmanager
 def openServer(cls=None, **kwa):
     """
-    Wrapper to create and open Server instances
+    Wrapper to create and open TCP Server instances
     When used in with statement block, calls .close() on exit of with block
 
     Parameters:
@@ -138,12 +136,15 @@ class Acceptor(tyming.Tymee):
             self.ss.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, self.bs)
 
         self.ss.setblocking(0) #non blocking socket
+        # bind to listen socket (host, port) to receive connections
+        self.ss.bind(self.ha)
+        self.ss.listen(5)
 
-        try:  # bind to listen socket (host, port) to receive connections
-            self.ss.bind(self.ha)
-            self.ss.listen(5)
-        except socket.error as ex:
-            return False
+        #try:  # bind to listen socket (host, port) to receive connections
+            #self.ss.bind(self.ha)
+            #self.ss.listen(5)
+        #except socket.error as ex:
+            #return False
 
         self.ha = self.ss.getsockname()  # get resolved ha after bind
         self.opened = True
